
#include "pw/core/debug.hpp"
#include "pw/visual/pipeline.hpp"
#include "pw/visual/shader.hpp"
#include "pw/visual/framebuffer.hpp"
#include "pw/core/size.hpp"

#include "runtime_lua.hpp"

namespace pw {

void register_visual_function(sol::state&,sol::table &ns)
{

    ns.new_usertype<pipeline>("pipeline"
							  ,"create",&pipeline::create
                              ,"draw",&pipeline::draw
                              );



    ns.new_usertype<shader>("shader"
                            ,sol::constructors<shader()>()
                            ,"ready",sol::readonly_property(&shader::ready)
                            ,"use",&shader::use
                            ,"build",&shader::build
                            ,"source",&shader::source
                            ,"set_source",&shader::set_source
                            ,"set_uniforms",&shader::set_uniforms
                            ,"set_uniform_float",&shader::set_uniform<float>
                            ,"set_uniform_mat4",&shader::set_uniform<matrix4x4&>
                            ,"set_uniform_vec4",&shader::set_uniform<vector4&>
                            ).new_enum<false>("shader_type"
                                       ,"fragment",shader::code_type::fragment
                                              ,"vertex",shader::code_type::vertex
                                              ,"geometry",shader::code_type::geometry
                                              ,"compute",shader::code_type::compute);


    ns.new_usertype<render_pass>("render_pass"
                                ,"submit",&render_pass::submit
                                 );

    ns.new_usertype<material>("material"
                              ,"color",sol::property(&material::_color));


    ns.new_usertype<mesh_renderer>("mesh_renderer"
                                   ,sol::constructors<mesh_renderer(),mesh_renderer(const geometry&)>()
                                   ,"create",&mesh_renderer::create
                                   ,"ready",sol::readonly_property(&mesh_renderer::ready)
                                   ,"release",&mesh_renderer::release
                                   ,"draw",&mesh_renderer::draw
                                   );

    ns.new_usertype<framebuffer>("framebuffer"
                                 ,sol::constructors<framebuffer()>()
                                 ,"create",&framebuffer::create
                                 ,"bind",&framebuffer::bind
                                 ,"unbind",&framebuffer::unbind
                                 ,"blit",&framebuffer::blit);
}

PW_REGISTER_LUA(visual)

}
